@@ -34,10 +34,11 @@
 #include "util.h"
 #include "main.h"
 #include "filter.h"
+#include "i18n.h"
 
 
 static int encode_simple( const char *filename, int mode );
-static int write_pubkey_enc_from_list( PKC_LIST pkc_list, DEK *dek, IOBUF out );
+static int write_pubkey_enc_from_list( PK_LIST pk_list, DEK *dek, IOBUF out );
 
 
 
@@ -81,7 +82,7 @@ encode_simple( const char *filename, int mode )
 
     /* prepare iobufs */
     if( !(inp = iobuf_open(filename)) ) {
-	log_error("can't open %s: %s\n", filename? filename: "[stdin]",
+	log_error(_("%s: can't open: %s\n"), filename? filename: "[stdin]",
 					strerror(errno) );
 	return G10ERR_OPEN_FILE;
     }
@@ -98,7 +99,7 @@ encode_simple( const char *filename, int mode )
 	    m_free(cfx.dek);
 	    m_free(s2k);
 	    iobuf_close(inp);
-	    log_error("error creating passphrase: %s\n", g10_errstr(rc) );
+	    log_error(_("error creating passphrase: %s\n"), g10_errstr(rc) );
 	    return rc;
 	}
     }
@@ -134,7 +135,7 @@ encode_simple( const char *filename, int mode )
 	pt->namelen = strlen(filename);
 	memcpy(pt->name, filename, pt->namelen );
 	if( !(filesize = iobuf_get_filelength(inp)) )
-	    log_info("warning: '%s' is an empty file\n", filename );
+	    log_info(_("%s: warning: empty file\n"), filename );
     }
     else { /* no filename */
 	pt = m_alloc( sizeof *pt - 1 );
@@ -185,24 +186,24 @@ encode_crypt( const char *filename, STRLIST remusr )
     cipher_filter_context_t cfx;
     armor_filter_context_t afx;
     compress_filter_context_t zfx;
-    PKC_LIST pkc_list;
+    PK_LIST pk_list;
 
     memset( &cfx, 0, sizeof cfx);
     memset( &afx, 0, sizeof afx);
     memset( &zfx, 0, sizeof zfx);
 
-    if( (rc=build_pkc_list( remusr, &pkc_list, 2)) )
+    if( (rc=build_pk_list( remusr, &pk_list, PUBKEY_USAGE_ENC)) )
 	return rc;
 
     /* prepare iobufs */
     if( !(inp = iobuf_open(filename)) ) {
-	log_error("can't open %s: %s\n", filename? filename: "[stdin]",
+	log_error(_("can't open %s: %s\n"), filename? filename: "[stdin]",
 					strerror(errno) );
 	rc = G10ERR_OPEN_FILE;
 	goto leave;
     }
     else if( opt.verbose )
-	log_info("reading from '%s'\n", filename? filename: "[stdin]");
+	log_info(_("reading from '%s'\n"), filename? filename: "[stdin]");
 
     if( !(out = open_outfile( filename, opt.armor? 1:0 )) ) {
 	rc = G10ERR_CREATE_FILE;  /* or user said: do not overwrite */
@@ -222,7 +223,7 @@ encode_crypt( const char *filename, STRLIST remusr )
     if( DBG_CIPHER )
 	log_hexdump("DEK is: ", cfx.dek->key, cfx.dek->keylen );
 
-    rc = write_pubkey_enc_from_list( pkc_list, cfx.dek, out );
+    rc = write_pubkey_enc_from_list( pk_list, cfx.dek, out );
     if( rc  )
 	goto leave;
 
@@ -232,7 +233,7 @@ encode_crypt( const char *filename, STRLIST remusr )
 	pt->namelen = strlen(filename);
 	memcpy(pt->name, filename, pt->namelen );
 	if( !(filesize = iobuf_get_filelength(inp)) )
-	    log_info("warning: '%s' is an empty file\n", filename );
+	    log_info(_("%s: warning: empty file\n"), filename );
     }
     else { /* no filename */
 	pt = m_alloc( sizeof *pt - 1 );
@@ -242,6 +243,7 @@ encode_crypt( const char *filename, STRLIST remusr )
     pt->timestamp = make_timestamp();
     pt->mode = 'b';
     pt->len = filesize;
+    pt->new_ctb = !pt->len && !opt.rfc1991;
     pt->buf = inp;
     init_packet(&pkt);
     pkt.pkttype = PKT_PLAINTEXT;
@@ -269,7 +271,7 @@ encode_crypt( const char *filename, STRLIST remusr )
 	pt->buf = NULL;
     free_packet(&pkt);
     m_free(cfx.dek);
-    release_pkc_list( pkc_list );
+    release_pk_list( pk_list );
     return rc;
 }
 
@@ -297,7 +299,7 @@ encrypt_filter( void *opaque, int control,
 		log_hexdump("DEK is: ",
 			     efx->cfx.dek->key, efx->cfx.dek->keylen );
 
-	    rc = write_pubkey_enc_from_list( efx->pkc_list, efx->cfx.dek, a );
+	    rc = write_pubkey_enc_from_list( efx->pk_list, efx->cfx.dek, a );
 	    if( rc )
 		return rc;
 
@@ -318,33 +320,35 @@ encrypt_filter( void *opaque, int control,
 
 
 /****************
- * Write pubkey-enc packets from the list of PKCs to OUT.
+ * Write pubkey-enc packets from the list of PKs to OUT.
  */
 static int
-write_pubkey_enc_from_list( PKC_LIST pkc_list, DEK *dek, IOBUF out )
+write_pubkey_enc_from_list( PK_LIST pk_list, DEK *dek, IOBUF out )
 {
     PACKET pkt;
-    PKT_public_cert *pkc;
+    PKT_public_key *pk;
     PKT_pubkey_enc  *enc;
     int rc;
 
-    for( ; pkc_list; pkc_list = pkc_list->next ) {
+    for( ; pk_list; pk_list = pk_list->next ) {
 	MPI frame;
 
-	pkc = pkc_list->pkc;
+	pk = pk_list->pk;
+	if( is_RSA(pk->pubkey_algo) )
+	    do_not_use_RSA();
 	enc = m_alloc_clear( sizeof *enc );
-	enc->pubkey_algo = pkc->pubkey_algo;
-	keyid_from_pkc( pkc, enc->keyid );
-	frame = encode_session_key( dek, pubkey_nbits( pkc->pubkey_algo,
-							  pkc->pkey ) );
-	rc = pubkey_encrypt( pkc->pubkey_algo, enc->data, frame, pkc->pkey );
+	enc->pubkey_algo = pk->pubkey_algo;
+	keyid_from_pk( pk, enc->keyid );
+	frame = encode_session_key( dek, pubkey_nbits( pk->pubkey_algo,
+							  pk->pkey ) );
+	rc = pubkey_encrypt( pk->pubkey_algo, enc->data, frame, pk->pkey );
 	mpi_free( frame );
 	if( rc )
 	    log_error("pubkey_encrypt failed: %s\n", g10_errstr(rc) );
 	else {
 	    if( opt.verbose ) {
 		char *ustr = get_user_id_string( enc->keyid );
-		log_info("%s encrypted for: %s\n",
+		log_info(_("%s encrypted for: %s\n"),
 		    pubkey_algo_to_string(enc->pubkey_algo), ustr );
 		m_free(ustr);
 	    }
