@@ -42,6 +42,10 @@ free_pubkey_enc( PKT_pubkey_enc *enc )
 {
     int n, i;
     n = pubkey_get_nenc( enc->pubkey_algo );
+    if( !n ) {
+	m_free(enc->data[0]);
+	enc->data[0] = NULL;
+    }
     for(i=0; i < n; i++ )
 	mpi_free( enc->data[i] );
     m_free(enc);
@@ -52,6 +56,10 @@ free_seckey_enc( PKT_signature *sig )
 {
     int n, i;
     n = pubkey_get_nenc( sig->pubkey_algo );
+    if( !n ) {
+	m_free(sig->data[0]);
+	sig->data[0] = NULL;
+    }
     for(i=0; i < n; i++ )
 	mpi_free( sig->data[i] );
     m_free(sig->hashed_data);
@@ -62,26 +70,46 @@ free_seckey_enc( PKT_signature *sig )
 
 
 void
-release_public_cert_parts( PKT_public_cert *cert )
+release_public_key_parts( PKT_public_key *pk )
 {
     int n, i;
-    n = pubkey_get_npkey( cert->pubkey_algo );
+    n = pubkey_get_npkey( pk->pubkey_algo );
+    if( !n ) {
+	m_free(pk->pkey[0]);
+	pk->pkey[0] = NULL;
+    }
     for(i=0; i < n; i++ ) {
-	mpi_free( cert->pkey[i] );
-	cert->pkey[i] = NULL;
+	mpi_free( pk->pkey[i] );
+	pk->pkey[i] = NULL;
     }
 }
 
 
 void
-free_public_cert( PKT_public_cert *cert )
+free_public_key( PKT_public_key *pk )
 {
-    release_public_cert_parts( cert );
-    m_free(cert);
+    release_public_key_parts( pk );
+    m_free(pk);
 }
 
-PKT_public_cert *
-copy_public_cert( PKT_public_cert *d, PKT_public_cert *s )
+static void *
+cp_fake_data( MPI a )
+{
+    byte *d, *s;
+    u16 len;
+
+    if( !a )
+	return NULL;
+    s = (byte*)a;
+    len = (s[0] << 8) | s[1];
+    d = m_alloc( len+2 );
+    memcpy(d, s, len+2);
+    return d;
+}
+
+
+PKT_public_key *
+copy_public_key( PKT_public_key *d, PKT_public_key *s )
 {
     int n, i;
 
@@ -89,32 +117,40 @@ copy_public_cert( PKT_public_cert *d, PKT_public_cert *s )
 	d = m_alloc(sizeof *d);
     memcpy( d, s, sizeof *d );
     n = pubkey_get_npkey( s->pubkey_algo );
-    for(i=0; i < n; i++ )
-	d->pkey[i] = mpi_copy( s->pkey[i] );
+    if( !n )
+	d->pkey[0] = cp_fake_data(s->pkey[0]);
+    else {
+	for(i=0; i < n; i++ )
+	    d->pkey[i] = mpi_copy( s->pkey[i] );
+    }
     return d;
 }
 
 void
-release_secret_cert_parts( PKT_secret_cert *cert )
+release_secret_key_parts( PKT_secret_key *sk )
 {
     int n, i;
 
-    n = pubkey_get_nskey( cert->pubkey_algo );
+    n = pubkey_get_nskey( sk->pubkey_algo );
+    if( !n ) {
+	m_free(sk->skey[0]);
+	sk->skey[0] = NULL;
+    }
     for(i=0; i < n; i++ ) {
-	mpi_free( cert->skey[i] );
-	cert->skey[i] = NULL;
+	mpi_free( sk->skey[i] );
+	sk->skey[i] = NULL;
     }
 }
 
 void
-free_secret_cert( PKT_secret_cert *cert )
+free_secret_key( PKT_secret_key *sk )
 {
-    release_secret_cert_parts( cert );
-    m_free(cert);
+    release_secret_key_parts( sk );
+    m_free(sk);
 }
 
-PKT_secret_cert *
-copy_secret_cert( PKT_secret_cert *d, PKT_secret_cert *s )
+PKT_secret_key *
+copy_secret_key( PKT_secret_key *d, PKT_secret_key *s )
 {
     int n, i;
 
@@ -122,8 +158,12 @@ copy_secret_cert( PKT_secret_cert *d, PKT_secret_cert *s )
 	d = m_alloc(sizeof *d);
     memcpy( d, s, sizeof *d );
     n = pubkey_get_nskey( s->pubkey_algo );
-    for(i=0; i < n; i++ )
-	d->skey[i] = mpi_copy( s->skey[i] );
+    if( !n )
+	d->skey[0] = cp_fake_data(s->skey[0]);
+    else {
+	for(i=0; i < n; i++ )
+	    d->skey[i] = mpi_copy( s->skey[i] );
+    }
     return d;
 }
 
@@ -208,11 +248,13 @@ free_packet( PACKET *pkt )
       case PKT_SYMKEY_ENC:
 	free_symkey_enc( pkt->pkt.symkey_enc );
 	break;
-      case PKT_PUBLIC_CERT:
-	free_public_cert( pkt->pkt.public_cert );
+      case PKT_PUBLIC_KEY:
+      case PKT_PUBLIC_SUBKEY:
+	free_public_key( pkt->pkt.public_key );
 	break;
-      case PKT_SECRET_CERT:
-	free_secret_cert( pkt->pkt.secret_cert );
+      case PKT_SECRET_KEY:
+      case PKT_SECRET_SUBKEY:
+	free_secret_key( pkt->pkt.secret_key );
 	break;
       case PKT_COMMENT:
 	free_comment( pkt->pkt.comment );
@@ -240,7 +282,7 @@ free_packet( PACKET *pkt )
  * Returns 0 if they match.
  */
 int
-cmp_public_certs( PKT_public_cert *a, PKT_public_cert *b )
+cmp_public_keys( PKT_public_key *a, PKT_public_key *b )
 {
     int n, i;
 
@@ -252,6 +294,8 @@ cmp_public_certs( PKT_public_cert *a, PKT_public_cert *b )
 	return -1;
 
     n = pubkey_get_npkey( b->pubkey_algo );
+    if( !n )
+	return -1; /* can't compare due to unknown algorithm */
     for(i=0; i < n; i++ ) {
 	if( mpi_cmp( a->pkey[i], b->pkey[i] ) )
 	    return -1;
@@ -264,20 +308,22 @@ cmp_public_certs( PKT_public_cert *a, PKT_public_cert *b )
  * Returns 0 if they match.
  */
 int
-cmp_public_secret_cert( PKT_public_cert *pkc, PKT_secret_cert *skc )
+cmp_public_secret_key( PKT_public_key *pk, PKT_secret_key *sk )
 {
     int n, i;
 
-    if( pkc->timestamp != skc->timestamp )
+    if( pk->timestamp != sk->timestamp )
 	return -1;
-    if( pkc->valid_days != skc->valid_days )
+    if( pk->valid_days != sk->valid_days )
 	return -1;
-    if( pkc->pubkey_algo != skc->pubkey_algo )
+    if( pk->pubkey_algo != sk->pubkey_algo )
 	return -1;
 
-    n = pubkey_get_npkey( pkc->pubkey_algo );
+    n = pubkey_get_npkey( pk->pubkey_algo );
+    if( !n )
+	return -1; /* can't compare due to unknown algorithm */
     for(i=0; i < n; i++ ) {
-	if( mpi_cmp( pkc->pkey[i] , skc->skey[i] ) )
+	if( mpi_cmp( pk->pkey[i] , sk->skey[i] ) )
 	    return -1;
     }
     return 0;
