@@ -532,8 +532,12 @@ get_seckey_byname( PKT_secret_key *sk, const char *name, int unprotect )
 {
     int rc;
 
-    rc = name ? key_byname( 1, NULL, sk, name )
-	      : lookup_sk( sk, 15, NULL, NULL );
+    if( !name && opt.def_secret_key && *opt.def_secret_key )
+	rc = key_byname( 1, NULL, sk, opt.def_secret_key );
+    else if( !name ) /* use the first one as default key */
+	rc = lookup_sk( sk, 15, NULL, NULL );
+    else
+	rc = key_byname( 1, NULL, sk, name );
     if( !rc && unprotect )
 	rc = check_secret_key( sk );
 
@@ -755,6 +759,15 @@ lookup( PKT_public_key *pk, int mode,  u32 *keyid,
 			size_t an;
 			byte *afp = fingerprint_from_pk(
 					k->pkt->pkt.public_key, &an );
+
+			if( DBG_CACHE ) {
+			    u32 aki[2];
+			    keyid_from_pk( k->pkt->pkt.public_key, aki );
+			    log_debug("         aki=%08lx%08lx algo=%d mode=%d an=%u\n",
+					    (ulong)aki[0], (ulong)aki[1],
+				    k->pkt->pkt.public_key->pubkey_algo,
+							mode, an );
+			}
 			if( an == mode && !memcmp( afp, name, an)
 			    && ( !pk->pubkey_algo
 				 || pk->pubkey_algo
