@@ -82,9 +82,7 @@ typedef struct {
     int  bufcount;
     int  secure;
     FILE  *debug;
-    int guard1;
     struct md_digest_list_s *list;
-    int guard2;
 } *MD_HANDLE;
 
 
@@ -123,6 +121,8 @@ void md_stop_debug( MD_HANDLE a );
 		    md_write( (h), NULL, 0 );		    \
 		(h)->buffer[(h)->bufcount++] = (c) & 0xff;  \
 	    } while(0)
+/*-- rmd160.c --*/
+void rmd160_hash_buffer( char *outbuf, const char *buffer, size_t length );
 
 
 /*-- cipher.c --*/
