@@ -72,6 +72,14 @@ void g10_log_mpidump( const char *text, MPI a );
   void g10_log_error( const char *fmt, ... ) __attribute__ ((format (printf,1,2)));
   void g10_log_info( const char *fmt, ... )  __attribute__ ((format (printf,1,2)));
   void g10_log_debug( const char *fmt, ... ) __attribute__ ((format (printf,1,2)));
+  void g10_log_fatal_f( const char *fname, const char *fmt, ... )
+			    __attribute__ ((noreturn, format (printf,2,3)));
+  void g10_log_error_f( const char *fname, const char *fmt, ... )
+			    __attribute__ ((format (printf,2,3)));
+  void g10_log_info_f( const char *fname, const char *fmt, ... )
+			    __attribute__ ((format (printf,2,3)));
+  void g10_log_debug_f( const char *fname,  const char *fmt, ... )
+			    __attribute__ ((format (printf,2,3)));
   #define BUG() g10_log_bug0(  __FILE__ , __LINE__, __FUNCTION__ )
 #else
   void g10_log_bug( const char *fmt, ... );
@@ -80,6 +88,10 @@ void g10_log_mpidump( const char *text, MPI a );
   void g10_log_error( const char *fmt, ... );
   void g10_log_info( const char *fmt, ... );
   void g10_log_debug( const char *fmt, ... );
+  void g10_log_fatal_f( const char *fname, const char *fmt, ... );
+  void g10_log_error_f( const char *fname, const char *fmt, ... );
+  void g10_log_info_f( const char *fname, const char *fmt, ... );
+  void g10_log_debug_f( const char *fname, const char *fmt, ... );
   #define BUG() g10_log_bug0( __FILE__ , __LINE__ )
 #endif
 
@@ -91,7 +103,10 @@ void g10_log_mpidump( const char *text, MPI a );
 #define log_error   g10_log_error
 #define log_info    g10_log_info
 #define log_debug   g10_log_debug
-
+#define log_fatal_f g10_log_fatal_f
+#define log_error_f g10_log_error_f
+#define log_info_f  g10_log_info_f
+#define log_debug_f g10_log_debug_f
 
 
 /*-- errors.c --*/
